/*
 * Game Chooser
 * (c) 2004 Graeme Sheppard
 * This code is licenced under the GNU GPL version 2
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.*;


/*
 * Game Chooser - pick a game, your video settings and rumble
 */
public class GameChooser extends JFrame implements ActionListener {
  // launcher is a sh script which makes settings and runs the game -
  // because java 1.5 seemingly won't set environment variables
  static String launcher = "./launcher";

  // the text file of games
  static String gamesFile = "gamelist";
  // columnn definitions of gamesFile
  static int nameColumn = 0;
  static int execColumn = 1;
  static int fsaaColumn = 2;
  static int aitfColumn = 3;

  // games is the main structure: names, settings etc
  static ArrayList<String[]> games = new ArrayList<String[]>();
  static String[] gameList;
  static String[] fsaaList = {
    "0: None",
    "1: 2x Bilinear Multisampling",
    "2: 2x Quincunx Multisampling",
    "3: None",
    "4: 4x Bilinear Multisampling",
    "5: 4x Gaussian Multisampling",
    "6: 2x Bilinear Multisampling by 4x Supersampling",
    "7: 4x Bilinear Multisampling by 4x Supersampling",
  };
  static String[] aitfList = {"None", "Low", "Medium", "High"};
    
  JPanel mainPanel;
  static JComboBox gameCB;
  static JComboBox fsaaCB;
  static JComboBox aitfCB;

  static int box_x = 400;
  static int box_y = 30;
  
  public GameChooser() {
    super("Game Chooser 0.10");
    setSize(550, 200);
    setBackground(Color.gray);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    
    generateGameArray(); // read in data

    mainPanel = new JPanel();
    mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
    Border edges = BorderFactory.createEmptyBorder(5, 5, 5, 5);
    mainPanel.setBorder(edges);
    getContentPane().add(mainPanel);

    gameCB = new JComboBox(gameList); // populate the drop-down boxes
    fsaaCB = new JComboBox(fsaaList);
    aitfCB = new JComboBox(aitfList);

    addCombo("Game", gameCB);
    addCombo("Fullscreen Antialiasing", fsaaCB);
    addCombo("Anisotropic Texture Filtering", aitfCB);
    
    JPanel subPanel = new JPanel();
    subPanel.setLayout(new BoxLayout(subPanel, BoxLayout.X_AXIS));
    subPanel.setBorder(edges);
    JButton button;
    button = new JButton("Play");
    button.addActionListener(this);
    subPanel.add(button);
    subPanel.add(Box.createRigidArea(new Dimension(10,0)));
    button = new JButton("Monitor");
    button.addActionListener(this);
    subPanel.add(button);
    subPanel.add(Box.createHorizontalGlue());
    button = new JButton("Quit");
    button.addActionListener(this);
    subPanel.add(button);
    
    mainPanel.add(subPanel);
    chooseGame(0);
  }
 
  // this reads in your list of games and favourite settings 
  void generateGameArray() {
    int index, gameCount, lineCount;
    BufferedReader input = null;
    String line = null;
    String[] array;
    
    try {
      input = new BufferedReader(new FileReader(new File(gamesFile)));
      lineCount = 0;
      while ((line = input.readLine()) != null)
      {
        lineCount++;
        if (line.equals("") || line.startsWith("#"))
          continue;
        array = line.split(",");
        if (array.length != 4)
          System.out.println("Error line " + lineCount + ": " + line);
        else
          games.add(array);
      }
      input.close();
    }
    catch (IOException err) {
        err.printStackTrace();
    }

    // extract the game name column so it can be used in the drop-down list
    gameCount = games.size();
    gameList = new String[gameCount];
    for (index = 0; index < gameCount; index++)
      gameList[index] = getGridValue(index, nameColumn);
  }
  
  // quick code to get values out of the structure
  String getGridValue(int y, int x) {
    String[] array = games.get(y);
    return array[x];
  }

  void addCombo(String description, JComboBox cb)  {
    JPanel subPanel = new JPanel();
    subPanel.setLayout(new BoxLayout(subPanel, BoxLayout.X_AXIS));

    JLabel label = new JLabel(description + ":");
    subPanel.add(label);    
    subPanel.add(Box.createHorizontalGlue());
    cb.setMaximumSize(new Dimension(box_x, box_y));
    cb.addActionListener(this);
    cb.setActionCommand(description);
    subPanel.add(cb);
    Border edges = BorderFactory.createEmptyBorder(5, 5, 5, 5);
    subPanel.setBorder(edges);
    mainPanel.add(subPanel); 
  }

  void chooseGame(int index) {
    gameCB.setSelectedIndex(index);
    // when a game is selected, these 2 lines remember your favourite
    // settings for it
    // comment out if you don't like it
    fsaaCB.setSelectedIndex(Integer.parseInt(getGridValue(index, fsaaColumn)));
    aitfCB.setSelectedIndex(Integer.parseInt(getGridValue(index, aitfColumn)));
  }
    
  public void actionPerformed(ActionEvent event) {
    String command = event.getActionCommand();
    if (command == "Quit")
      System.exit(0);
    else if (command == "Game")
      chooseGame(gameCB.getSelectedIndex());
    else if (command == "Play")
    {
      try {
        playGame();
      }
      catch (Exception err) {
      }
    }
    else if (command == "Monitor")
      runProgram("nvidia-settings");
  }

  void playGame() {
    Integer index;
    String cmdline; // build a command line in this variable
    
    cmdline = launcher + " ";
    index = fsaaCB.getSelectedIndex();
    cmdline += index.toString() + " ";
    index = aitfCB.getSelectedIndex();
    cmdline += index.toString() + " ";
    cmdline += getGridValue(gameCB.getSelectedIndex(), execColumn);
    // cmdline might look like "launcher 2 0 quake3" at this point
    runProgram("xterm -e /bin/sh " + cmdline);
  }
  
  void runProgram(String cmdline)
  {
    try {
      Runtime.getRuntime().exec(cmdline);
    } 
    catch (Exception err) {
     err.printStackTrace();
    }
  }
   
  public static void main(String args[]) {
    GameChooser mainFrame = new GameChooser();
    mainFrame.setVisible(true);
  }
}

